.module       MIDI;

.include    <system.k>;
.include    <flashint.k>;
.include    <flashdrv.k>;

.global         process_midi;

.global         inputmidi;

.external       nSend;

.external       cur_setting;
.external       brain_delta_hi,brain_delta_lo,delta_hi,delta_lo;
.external       cur_write_setting;
.external       volume,cur_volume;
.external       dry_volume,cur_dry_volume;
.external       reverse,cur_reverse;
.external       distortion,cur_distortion;
.external       sample,cur_sample;
.external       retrigger,cur_retrigger,brain_pos_hi,brain_pos_lo;
.external       effectfb,cur_effectfb;
.external       inplevel,cur_inplevel;
.external       portamento,cur_portamento,porta_step;
.external       target_delta_hi,target_delta_lo;
.external       brain_size,cur_brain_size;
.external       decay,decay_level,cur_decay,decay_step;
.external       pan_l,pan_r,cur_l_pan,cur_r_pan;
.external       inp_source,cur_inp_source;
.external       mod_wave_delta,mod_wave_ptr,mod_deltas;

{ variables needed to interpret several MIDI commands }
.var    state,                  { MIDI state machine }
        midi_ch,                { MIDI channel }
        midi_vel,               { MIDI velocity }
        midi_note,              { MIDI note }
        midi_cont,              { MIDI controller number }
        midi_val,               { MIDI controller value }
        inputmidi,              { MIDI input data from serial port }
        midi_trash;             { MIDI garbage received }

.init state: 0x01;

{ variables used for reading pot's }
.var    pot1,
        pot2,
        pot3,
        pot4;

{ These are for the tuned pitch shifter -------------------------------- }

.var
        deltalo[80],deltahi[80];
.include <deltahi.h>;
.include <deltalo.h>;

{ These are for the tuned hold buffer ---------------------------------- }

.var
        holdbuf_len[120];
.include <holdbuf.h>;

{ These are for the tuned modulation ----------------------------------- }
.var
    freqtab[128];               { frequency table }
.include <freqtab.h>;

{ Waveform to modulate with }
.var    
    wave_squ[256];              { triangle wave }
.include <wave_squ.h>;    
.global wave_squ;


{ let the cartooooooooons, begin! -------------------------------------- }
process_midi:   

        { determine "state" of state machine }
        ax0=dm(state);ay0=1;ar=ax0-ay0;if eq jump idle1;
        
        { MIDI Note On messages }
        ax0=dm(state);ay0=3;ar=ax0-ay0;if eq jump gotnote;
        ax0=dm(state);ay0=4;ar=ax0-ay0;if eq jump gotvel;

        { MIDI Note Off messages }
        ax0=dm(state);ay0=7;ar=ax0-ay0;if eq jump noteoff2;
        ax0=dm(state);ay0=8;ar=ax0-ay0;if eq jump noteoff3;

        { MIDI Controller messages }
        ax0=dm(state);ay0=5;ar=ax0-ay0;if eq jump contr2;
        ax0=dm(state);ay0=6;ar=ax0-ay0;if eq jump contr3;

        jump end_proc_midi;

end_proc_midi:
        rts;

{ idle! }
idle1:
        i4 = dm (PTR_TO_GET_CHAR);call (i4);
        if lt jump end_proc_midi; { time out }

{        ay0 = 90; none = ax1 - ay0;if eq rts;} { go back to monitor. }

        dm(inputmidi) = ax1;

        { filter out MIDI commands }
        ay1 = ax1;              { save copy of original byte in ay1 }
        ax0 = 0x00f0;
        ar  = ax0 and ay1;
        ax1 = ar;               { ax1 = MIDI command type }

        { look for MIDI note on messages }
        ay0 = 0x0090;{ note on =1001xxxx} none = ax1 - ay0;if eq jump noteon1;

        { look for MIDI note off messages }
        ay0 = 0x0080;{ note on =1000xxxx} none = ax1 - ay0;if eq jump noteoff1;

        { look for MIDI Controller messages on any channel }
        ay0 = 0x00b0;{ control =1011xxxx} none = ax1 - ay0;if eq jump contr1;

        { print an 'X' if any other byte code received }
        dm(midi_trash) = ay1;
        ax1 = 88;            call nSend;
        ay1 = dm(midi_trash);call nSend;

        jump end_proc_midi;

{ handle MIDI Note on message --------------------------------------- }
noteon1:         
        ax1 = 0x000f;
        ar  = ax1 and ay1;
        ay1 = 0x01;
        ar  = ar + ay1; 
        dm(midi_ch) = ar;               { save midi channel number }
        
        ax1 = 111; { 'o' } call nSend;ax1 = 110; { 'n' } call nSend;
        ay0 = 0x31;
        ax1 = dm(midi_ch);
        ar  = ax1 + ay0;ax1  = ar;call nSend;{ print 'on#' for note on }
        ax1 = 0x20;call nSend;
        
        ar = 3;  dm(state)=ar;jump end_proc_midi;

gotvel: { MIDI velocity }

        i4 = dm (PTR_TO_GET_CHAR);call (i4);if lt jump end_proc_midi;
        dm(midi_vel)=ax1;

        ax1 = 118; { 'v' } call nSend;ax1 = 101; { 'e' } call nSend;
        ax1 = dm(midi_vel);call nSend;{ print 've' for velocity }
        ax1 = 0x20;call nSend;

        ar = 1;dm(state)=ar;jump end_proc_midi;

gotnote:{ MIDI note }

        i4 = dm(PTR_TO_GET_CHAR);call (i4);if lt jump end_proc_midi; { time out }
        dm(midi_note) = ax1;

        ax1 = 110; { 'n' } call nSend;ax1 = 111; { 'o' } call nSend;
        ax1 = dm(midi_note);call nSend;{ print 'no#' }
        ax1 = 0x0d;call nSend;

        call newsound;  { add sound to queue }

        ar = 4;dm(state)=ar;jump end_proc_midi;

{ handle MIDI Note Off message --------------------------------------- }
noteoff1:         
        ax1 = 0x000f;
        ar  = ax1 and ay1;
        ay1 = 1;
        ar  = ar + ay1;
        dm(midi_ch) = ar;               { save midi channel number }
        
        ax1 = 111; { 'o' } call nSend;ax1 = 102; { 'f' } call nSend;ax1 = 102;call nSend;
        ay0 = 0x31;
        ax1 = dm(midi_ch);
        ar  = ax1 + ay0;ax1  = ar;call nSend;{ print 'off#' for note on }
        ax1 = 0x20;call nSend;
        
        ar = 7;  dm(state)=ar;jump end_proc_midi;

noteoff2:
        i4 = dm (PTR_TO_GET_CHAR);call (i4);if lt jump end_proc_midi;
        dm(midi_note)=ax1;

        ax1 = 110; { 'n' } call nSend;ax1 = 111; { 'o' } call nSend;
        ax1 = dm(midi_note);call nSend;{ print 'no' for off note }
        ax1 = 0x20;call nSend;

        ar = 8;  dm(state)=ar;jump end_proc_midi;

noteoff3:

        ax1 = 118; { 'v' } call nSend;ax1 = 101; { 'e' } call nSend;
        ax1 = dm(midi_note);call nSend;{ print 've#' }
        ax1 = 13;call nSend;

        call killsound;  { add sound to queue }

        ar = 1;  dm(state)=ar;jump end_proc_midi;

{ handle MIDI Control message ------------------------------------ }
contr1: { MIDI control message }
        
        ax1 = 0x0f;
        ar  = ax1 and ay1;
        ay1 = 1;
        ar  = ax1 + ay1;
        dm(midi_ch)=ar;         { save midi channel number }

        { print "CO#" for first controller message byte }
jump skip_contr1;
        ax1 =  67; { 'C' } call nSend;
        ax1 =  79; { 'O' } call nSend;
skip_contr1:
        ax1 = dm(midi_ch); call nSend;

        ar = 5;dm(state)=ar;jump end_proc_midi;

contr2: { MIDI controller number }

        i4 = dm (PTR_TO_GET_CHAR);call (i4);if lt jump end_proc_midi;
        dm(midi_cont)=ax1;

        { print "NO#" for second controller message byte }
jump skip_contr2;
        ax1 =  78; { 'N' }  call nSend;
        ax1 =  79; { 'O' }  call nSend;
skip_contr2:
        ax1 = dm(midi_cont);call nSend;

        ar = 6;dm(state)=ar;jump end_proc_midi;

contr3: { MIDI control value }

        i4 = dm(PTR_TO_GET_CHAR);call (i4);if lt jump end_proc_midi;
        dm(midi_val)=ax1;

        { print "VA#" for third controller message byte }
jump skip_contr3;
        ax1 =  86; { 'V' } call nSend;
        ax1 =  65; { 'A' } call nSend;
skip_contr3:
        ax1 = dm(midi_val);call nSend;
        
        { dispatch appropriate subroutine for controller }
        ax1 = dm(midi_cont);
        ay1 =  0;none = ax1-ay1;if eq jump set_setting;
        ay1 =  1;none = ax1-ay1;if eq jump set_cur_write_setting;
        ay1 =  2;none = ax1-ay1;if eq jump set_cur_volume;
        ay1 =  3;none = ax1-ay1;if eq jump set_current_note;
        ay1 =  4;none = ax1-ay1;if eq jump set_reverse;
        ay1 =  5;none = ax1-ay1;if eq jump set_distortion;
        ay1 =  6;none = ax1-ay1;if eq jump set_sample;
        ay1 =  7;none = ax1-ay1;if eq jump set_retrigger;
        ay1 =  8;none = ax1-ay1;if eq jump set_portamento;
        ay1 =  9;none = ax1-ay1;if eq jump set_brain_size;
        ay1 = 10;none = ax1-ay1;if eq jump set_effectfb;
        ay1 = 11;none = ax1-ay1;if eq jump set_dry_volume;
        ay1 = 12;none = ax1-ay1;if eq jump set_decay;
        ay1 = 13;none = ax1-ay1;if eq jump set_inplevel;
        ay1 = 14;none = ax1-ay1;if eq jump set_pan;
        ay1 = 15;none = ax1-ay1;if eq jump set_inp_source;
        ay1 = 16;none = ax1-ay1;if eq jump set_mod_delta;

        { Special controller values for using flash file system }
        ay1 = 50;none = ax1-ay1;if eq jump set_addr_byte;
        ay1 = 51;none = ax1-ay1;if eq jump set_addr_byte;
        ay1 = 52;none = ax1-ay1;if eq jump set_addr_byte;
        ay1 = 53;none = ax1-ay1;if eq jump set_addr_byte;

        ay1 = 54;none = ax1-ay1;if eq jump FLASH_READ;
        ay1 = 55;none = ax1-ay1;if eq jump FLASH_WRITE;
        ay1 = 56;none = ax1-ay1;if eq jump FLASH_ERASE;
        ay1 = 57;none = ax1-ay1;if eq jump FLASH_IDENT;
        ay1 = 58;none = ax1-ay1;if eq jump FLASH_P_ADDR;
        ay1 = 59;none = ax1-ay1;if eq jump set_data_byte;
        ay1 = 60;none = ax1-ay1;if eq jump FLASH_P_DATA;

        { Extra messages for reading I/O ports }
        ay1 = 70;none = ax1-ay1;if eq jump READ_IO;
        ay1 = 71;none = ax1-ay1;if eq jump READ_PFTYPE;
        ay1 = 72;none = ax1-ay1;if eq jump WRITE_PFTYPE;
        ay1 = 73;none = ax1-ay1;if eq jump READ_PFDATA;
        ay1 = 74;none = ax1-ay1;if eq jump READ_PFDATA;

{ These will be brought out to flashint.dsp soon ------------- }
{ useful...it prints out [Addr3][Addr2][Addr1][Addr0] }
FLASH_P_ADDR:
        m0  = -1;
        l0  = 0;

        ax0 = 0x03;
        ay0 = ^addr_byte;
        ar  = ax0 + ay0;
        i0  = ar;
        ax1 = dm(i0,m0);  call nSend;   { print Addr 3 }

        ax0 = 0x02;
        ay0 = ^addr_byte;
        ar  = ax0 + ay0;
        i0  = ar;
        ax1 = dm(i0,m0);  call nSend;   { print Addr 2 }

        ax0 = 0x01;
        ay0 = ^addr_byte;
        ar  = ax0 + ay0;
        i0  = ar;
        ax1 = dm(i0,m0);  call nSend;   { print Addr 1 }

        ax0 = 0x00;
        ay0 = ^addr_byte;
        ar  = ax0 + ay0;
        i0  = ar;
        ax1 = dm(i0,m0);  call nSend;   { print Addr 0 }

        jump contr3_done;               { ciao                        }

{ useful...it prints out [Addr3][Addr2][Addr1][Addr0] }
FLASH_P_DATA:
        ax1 = dm(data_byte);  call nSend;   { print Addr 3 }

        jump contr3_done;               { ciao                        }


FLASH_IDENT:
        ax1 = FLASH_MEM_IDENT;
        dm(c_byte) = ax1;               { flash driver command number }
        call assemble_addr;             { assemble addr               }
        call mem_ident;                 { call flash_driver function  }
        ax1 = dm(d_byte);call nSend;    { print out results           }
        jump contr3_done;               { ciao                        }

FLASH_WRITE:
        ax1 = FLASH_PRG_BYTE;
        dm(c_byte) = ax1;               { flash command               }
        ax1 = dm(data_byte);
        dm(d_byte) = ax1;               { 8-bit value to write        } 
        call assemble_addr;

        call prog_byte;                 { call flash driver           }

        jump contr3_done;               
        
FLASH_READ:
        call assemble_addr;
        call read_byte;                 { call flash driver           }
        ax1 = dm(d_byte);call nSend;    { print out returned data     }

        jump contr3_done;               { Seeya                       }

{ How much fucking memory can this thing access? 4 MB! ------- }
FLASH_ERASE:
        ax1 = FLASH_ERASE_SECT;         { flash driver command }
        dm(c_byte) = ax1;               { sector to erase? }
        call assemble_addr;
        call sect_erase;                { call flash driver     }

        jump contr3_done;               { ciao                  }

READ_IO:
        ax1 = io(0x000); dm(pot1)=ax1; call nSend;
        ax1 = io(0x100); dm(pot2)=ax1; call nSend;
        ax1 = io(0x200); dm(pot3)=ax1; call nSend;
        ax1 = io(0x300); dm(pot4)=ax1; call nSend;

        jump contr3_done;

READ_PFTYPE:
        ax1 = dm(PFTYPE);
        call nSend;

        jump contr3_done;

WRITE_PFTYPE:
        ax1 =dm(midi_val);
        dm(PFTYPE)=ax1;

        jump contr3_done;

READ_PFDATA:
        ax1 = dm(PFDATA);
        call nSend;

        jump contr3_done;

WRITE_PFDATA:
        ax1 =dm(midi_val);
        dm(PFDATA)=ax1;

        jump contr3_done;


assemble_addr:
        i0  = ^addr_byte;
        m0  = 1;
        l0  = 0;
        ay0 = dm(i0,m0);                { read addr0 }
        sr0 = dm(i0,m0);                { read addr1 }
        sr  = lshift sr0 by 8 (lo);     { combine 'em }
        ar  = sr0 or ay0;
        dm(addr_lo) = ar;               { write low 16-bits of address }

        ay0 = dm(i0,m0);                { read addr2 }
        sr0 = dm(i0,m0);                { read addr3 }
        sr  = lshift sr0 by 8 (lo);     { combine 'em }
        ar  = sr0 or ay0;
        dm(addr_hi) = ar;               { write high 16-bits of address }

        jump contr3_done;

set_addr_byte:
        { remove controller message offset of 50 to leave index into addr_byte }
        ax1 = 50;
        ar  = ay1 - ax1;
        ay1 = ar;

        m0  = 0;
        l0  = 0;
        ay0 = ^addr_byte;
        ax1 = 0x03;
        ar  = ax1 and ay1;
        ar  = ar  +   ay0;
        i0  = ar;
        ay1 = dm(midi_val);
        dm(i0,m0) = ay1;
        i0  = ^addr_byte;

        jump contr3_done;

set_data_byte:
        ay1 = dm(midi_val);
        dm(data_byte) = ay1;

        jump contr3_done;

contr3_done:

        { return to idle state }
        ar = 1;dm(state)=ar;
        jump end_proc_midi;

set_setting:
        ar  = dm(cur_retrigger);
        none = pass ar;
        if eq jump continue_copying;

do_retrigger:
        ax1 = 0x0000;
        dm(brain_pos_hi) = ax1;
        dm(brain_pos_lo) = ax1;

continue_copying:        
        ay1 = dm(midi_val);     { set it up }
        dm(cur_setting) = ay1;

        ar  = ^delta_lo;        { copy delta lo }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(target_delta_lo) = ax1;

        ar  = ^delta_hi;        { copy delta hi }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(target_delta_hi) = ax1;

        ar  = ^volume;          { copy effect volume }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_volume) = ax1;

        ar  = ^dry_volume;      { copy direct volume }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_dry_volume) = ax1;

        ar  = ^reverse;         { set reverse mode }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_reverse) = ax1;

        ar  = ^distortion;      { copy distortion level }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_distortion) = ax1;

        ar  = ^sample;          { copy sample flag }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_sample) = ax1;

        ar  = ^retrigger;       { copy retrigger flag }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_retrigger) = ax1;

        ar  = ^effectfb;       { copy effect feedback volume }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_effectfb) = ax1;

        ar  = ^inplevel;       { copy input level }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_inplevel) = ax1;

        ar  = ^brain_size;       { copy brain_size }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_brain_size) = ax1;

        ar  = ^decay;           { copy decay }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_decay) = ax1;

        ar  = ^pan_l;           { copy left pan }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_l_pan) = ax1;

        ar  = ^pan_r;           { copy right pan }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_r_pan) = ax1;

        ar  = ^inp_source;      { copy input source select }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(cur_inp_source) = ax1;

        ar  = ^mod_deltas;      { copy modulation wave delta }
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(i0,m0);
        dm(mod_wave_delta) = ax1;

        ax1 = dm(cur_portamento);       
        dm(porta_step) = ax1;           { reset portamento countdown timer }

        ax1 = 100;
        dm(decay_step) = ax1;           { reset decay countdown timer }

        ax1 = 0x7fff;
        dm(decay_level) = ax1;          { reset decay envelope level }

        jump contr3_done;

set_cur_write_setting:
        ay1 = dm(midi_val);
        dm(cur_write_setting) = ay1;

        jump contr3_done;

set_current_note:
        ar  = ^deltalo;
        ay1 = dm(midi_val);
        ar  = ar + ay1;
        i0  = ar;
        ax0 = dm(i0,m0);        { the deltalo value for this note }
        
        ar  = ^delta_lo;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;               { the delta_lo pos for current write setting }
        dm(i0,m0) = ax0;

        ar  = ^deltahi;
        ay1 = dm(midi_val);
        ar  = ar + ay1;
        i0  = ar;
        ax0 = dm(i0,m0);        { the deltahi value for this note }
        
        ar  = ^delta_hi;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;               { the delta_hi pos for current write setting }
        dm(i0,m0) = ax0;

        jump contr3_done;

set_cur_volume:
        ar  = ^volume;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;

        ax1 = dm(midi_val);
        ar  = ax1;
        sr  = lshift ar by 8(lo);
        ay1 = sr0;
        ar  = ax1 or ay1;
        dm(i0,m0) = ar;

        jump contr3_done;

set_dry_volume:
        ar  = ^dry_volume;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;

        ax1 = dm(midi_val);
        ar  = ax1;
        sr  = lshift ar by 8(lo);
        ay1 = sr0;
        ar  = ax1 or ay1;
        dm(i0,m0) = ar;

        jump contr3_done;

set_reverse:
        ar  = ^reverse;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(midi_val);
        dm(i0,m0) = ax1;

        jump contr3_done;

set_distortion:
        ar  = ^distortion;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;

        ax1 = dm(midi_val);     { distortion level must be at least 0x0800 to get unity output }
        ay1 = 0x08;
        ar  = ax1 + ay1;        { add 0x08 }

        sr  = lshift ar by 8(lo);       { multiply by 0x0100 }
        ay1 = sr0;
        dm(i0,m0) = ay1;

        jump contr3_done;

set_sample:
        ar  = ^sample;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(midi_val);
        dm(i0,m0) = ax1;

        jump contr3_done;

set_retrigger:
        ar  = ^retrigger;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(midi_val);
        dm(i0,m0) = ax1;

        jump contr3_done;

set_effectfb:
        ar  = ^effectfb;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;

        ax1 = dm(midi_val);
        ar  = ax1;
        sr  = lshift ar by 8(lo);
        ay1 = sr0;
        ar  = ax1 or ay1;
        dm(i0,m0) = ar;

        jump contr3_done;

set_inplevel:
        ar  = ^inplevel;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;

        ax1 = dm(midi_val);
        ar  = ax1;
        sr  = lshift ar by 8(lo);
        ay1 = sr0;
        ar  = ax1 or ay1;
        dm(i0,m0) = ar;

        jump contr3_done;

set_portamento:
        ar  = ^cur_portamento;
        i0  = ar;

        ax1 = dm(midi_val);
        ar  = ax1;
        sr  = lshift ar by 1(lo);
        ay1 = sr0;
        ar  = ax1 or ay1;
        dm(i0,m0) = ar;

        jump contr3_done;

set_brain_size:
        ax0 = ^holdbuf_len;
        ay0 = dm(midi_val);
        ar  = ax0 + ay0;
        i0  = ar;
        ax0 = dm(i0,m0);

        ar  = ^brain_size;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;
        dm(i0,m0) = ax0;
        
        jump contr3_done;

set_decay:
        ar  = ^decay;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(midi_val);
        dm(i0,m0) = ax1;

        jump contr3_done;

set_pan:
        ar  = ^pan_l;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;

        ar  = ^pan_r;
        ar  = ar + ay1;
        i1  = ar;

        ax0 = dm(midi_val);
        ar  = ax0;
        sr  = lshift ar by 8 (lo);
        ay0 = sr0;
        sr  = lshift ar by 1 (lo);
        ax0 = sr0;
        ar  = ax0 or ay0;
        ay0 = ar;
        dm(i0,m0) = ay0;

        ar  = 0x7fff;
        ar  = ar - ay0;
        ax0 = ar;
        dm(i1,m0) = ax0;

        jump contr3_done;

set_inp_source:
        ar  = ^inp_source;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;
        ax1 = dm(midi_val);
        dm(i0,m0) = ax1;

        jump contr3_done;

set_mod_delta:
        ar  = ^mod_deltas;
        ay1 = dm(cur_write_setting);
        ar  = ar + ay1;
        i0  = ar;
        ay1 = dm(midi_val);             { Look up 8.8 delta for note number }
        ar  = ^freqtab;
        ar  = ar + ay1;
        i1  = ar;
        ax1 = dm(i1,m0);
        dm(i0,m0) = ax1;

        jump contr3_done;


{ remove a sound from the sound queue ---------------------------------- }
killsound:
        
        rts;


{ Use MIDI note number to set pitch shifter ---------------------------- }
newsound:

        ar  = ^deltalo;
        ay1 = dm(midi_note);
        ar  = ar + ay1;
        i0  = ar;
        ax0 = dm(i0,m0);        { the deltalo value for this note }
        dm(target_delta_lo) = ax0;

        ar  = ^deltahi;
        ay1 = dm(midi_note);
        ar  = ar + ay1;
        i0  = ar;
        ax0 = dm(i0,m0);        { the deltahi value for this note }
        dm(target_delta_hi) = ax0;

        rts;

{ put number to be printed in ax1 }
{ uses ar, sr, ax0, ay0, ay1, etc }
hexprint:
        ay0 = ax1;
        ar  = ay0;sr  = lshift ar by -12 (lo);   call hexdigit;call nSend;
        ar  = ay0;sr  = lshift ar by -8 (lo);    call hexdigit;call nSend;
        ar  = ay0;sr  = lshift ar by -4 (lo);    call hexdigit;call nSend;
        ar  = ay0;sr  = lshift ar by 0 (lo);     call hexdigit;call nSend;
        ax1 = 0x20;call nSend;
        rts;

hexdigit:
        ay1 = 0x000f;
        ax0 = sr0;
        ar  = ax0 and ay1;      { separate the digit we want }

        ax0 = ar;
        ay1 = 0x0a;
        ar  = ax0 - ay1;
        if ge jump hex_letter;

hex_number:
        ay1 = 0x003a;
        ar  = ar + ay1;
        ax1 = ar;
        rts;

hex_letter:
        ay1 = 0x41;
        ar  = ar + ay1;
        ax1 = ar;
        rts;



.endmod;

